// Source file: c:/DopeWars/DopeWars/DopeClient.java

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;

import DopeWars.Framework.*;
import DopeWars.DopeWars.*;
import DopeWars.DopeWars.DataHolders.*;
import DopeWars.DopeWars.Exceptions.*;

/**
 * The DopeClient coordinates the gather of information from the client as
 * well as presenting choices.
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Phase 2.0 (Meat)
 */
public class DopeClient extends Frame implements IConnectorUser, WindowListener,
																								 IRequestTypes {
	  private CardLayout mainLayout;
    private GameSettingsD m_gameSettings;
    private UserSettingsD m_userSettings;
    private UserD m_user;

    // The Various Panels to show the user
    private UserSettingsPanel m_userSettingsPanel;
    private WaitingOnServerPanel m_waitingOnServerPanel;
    private LocationListPanel m_locationListPanel;
    private LocationPanel m_locationPanel;
    private ScorePanel m_scorePanel;

    private Connector m_connector;

	/**
	 * @roseuid 392606D20025
	 */
	public static void main(String args[]) {
		Frame f = new DopeClient();
		f.setSize(640,480);
		f.setLocation(200,200);
		f.setVisible(true);
	}

	DopeClient() {
		super("Dope Client");

		mainLayout = new CardLayout();
		setLayout(mainLayout);

		// The first panel that's going to go in is the User Settings Panel
		getUserSettings();

		addWindowListener(this);
	}

	public void windowClosing(WindowEvent we) {
		System.exit(0);
	}
	public void windowClosed(WindowEvent we) {}
	public void windowIconified(WindowEvent we) {}
	public void windowActivated(WindowEvent we) {}
	public void windowOpened(WindowEvent we) {}
	public void windowDeiconified(WindowEvent we) {}
	public void windowDeactivated(WindowEvent we) {}

	/**
		@roseuid 3925F6230363
	*/
	public void handleRequest(int requestType, Object requestObject, String requestor) {
		switch (requestType) {
			case GAME_START:
				// The server has issued the start of game command
				gameStart((GameSettingsD) requestObject);
				break;
			case NEXT_TURN:
				// The server has issued the next turn command
				nextTurn();
				break;
			case GIVE_LOCATION:
				// The server has given us a location
				showLocation( (LocationD) requestObject);
				break;
			case GAME_OVER:
				// The game is over cool
				endGame();
				break;
			case SCORE_LIST:
				showScoreList( (ScoreListD) requestObject);
				break;
		}
	}

	public void closeConnection(String requestor) {}

	private void showComponent(String key) {
		mainLayout.show(this, key);
	}

	/**
		@roseuid 3925F80F0162
	*/
	private void connectWithServer(String serverName) throws IOException {
		// Establish a connector to the server
		Socket socket = new Socket(serverName, 4440);
		m_connector = new Connector(socket, this);

		// Send the hello packet with the basic user settings
		m_connector.send(HELLO, m_userSettings);
	}

	/**
		@roseuid 3926073B0239
	*/
	private void getUserSettings() {
		m_userSettingsPanel = new UserSettingsPanel();
		add(m_userSettingsPanel, m_userSettingsPanel.name);
		showComponent(m_userSettingsPanel.name);
	}

	/**
		@roseuid 3926076A01E7
	*/
	private void showInitialSettings() {
	}

	/**
		@roseuid 392608D800F3
	*/
	private void showWaitingOnServer(String message) {
		if(m_waitingOnServerPanel == null) {
			m_waitingOnServerPanel = new WaitingOnServerPanel(message);
			add(m_waitingOnServerPanel, m_waitingOnServerPanel.name);
		} else {
			m_waitingOnServerPanel.setMessage(message);
		}
		showComponent(m_waitingOnServerPanel.name);
	}

    /**
       @roseuid 392609350395
     */
    public void gameStart(GameSettingsD gameSettings) {
			System.out.println("Game started!");
			m_gameSettings = gameSettings;

			// Prepare the UserD object appropriately
			m_user = new UserD(m_userSettings, m_gameSettings);

			showWaitingOnServer("Game Started.  Waiting for First Turn Start");
    }

    private void endGame() {
			showWaitingOnServer("Game Ended.  Waiting on Scores");
			m_connector.send(GAME_DONE, m_user);
		}

    private void nextTurn() {
			m_user.nextTurn(m_gameSettings);
			showLocationList();
		}

    /**
       @roseuid 3926CBD00100
     */
    private void showLocationList() {
			// Build the location panel if we need to, then display it to the user.
			if(m_locationListPanel == null) {
				m_locationListPanel = new LocationListPanel();
				add(m_locationListPanel, m_locationListPanel.name);
			};

			showComponent(m_locationListPanel.name);
    }

    /**
       @roseuid 3926D0DB0084
     */
    private void gotoLocation(String locationName) {
			showWaitingOnServer("Location Requested");
			m_connector.send(GET_LOCATION, locationName);
    }

    /**
       @roseuid 392734480107
     */
    private void showLocation(LocationD location) {
			if(m_locationPanel == null) {
				m_locationPanel = new LocationPanel();
				add(m_locationPanel, m_locationPanel.name);
			};

			m_locationPanel.update(location);

			showComponent(m_locationPanel.name);
    }

    /**
       @roseuid 392736BF01FE
     */
    public void turnOver() {
			showWaitingOnServer("Waiting for Next Turn");
			m_connector.send(TURN_DONE, m_user);
    }

    /**
       @roseuid 39273704013F
     */
    private void loanChange(int amount) {
    }

    /**
       @roseuid 3927374A033F
     */
    private void bankAmountChange(int amount) {
    }

    /**
       @roseuid 39273A9203B5
     */
    private void showScoreList(ScoreListD scoreList) {
			System.out.println("Showing score list");
			if(m_scorePanel == null) {
				m_scorePanel = new ScorePanel(scoreList);
				add(m_scorePanel, m_scorePanel.name);
			}

			showComponent(m_scorePanel.name);
    }

    /**
       @roseuid 39273AE70140
     */
    private void newGame() {
    }

		/**
		 * The waiting on server panel just shows the user stuff
		 *
		 **/
		private class WaitingOnServerPanel extends Panel {
			private Label m_messageLabel;
			public String name = "WaitingOnServerPanel";
			public WaitingOnServerPanel(String errorMessage) {
				System.out.println("Waiting " + errorMessage);
				// setLayout(new GridLayout(2,1));
				add(new Label("Waiting on Server..."));
				m_messageLabel = new Label(errorMessage);
				add(m_messageLabel);
			}

			public void setMessage(String errorMessage) {
				m_messageLabel.setText(errorMessage);
			}

		}

		/**
		 * The panel that allows a user to find a location
		 */
		private class LocationPanel extends Panel implements ActionListener {
			public String name = "LocationPanel";
			private LocationD currLocation;
			private Button amountArray[];
			private Button priceArray[];
			private TextField amountField;
			private Label moneyLabel;
			private Label pocketLabel;
			private Label loanLabel;

			public LocationPanel() {
				// Okay, we've got a the list of drugs... let's build!
				DrugSettingsD tempDrugSetting;
				Button tempButton;
				int Kounter = 0;
				ArrayList drugSettingList = m_gameSettings.getDrugSettingList();
				int drugCount = drugSettingList.size();
				setLayout(new GridLayout(drugCount+3, 3));
				add(new Label("Amount", Label.CENTER));
				add(new Label("Drug", Label.CENTER));
				add(new Label("Price", Label.CENTER));

				// Create the Amount Array and Price Array of buttons at the
				//  size of the number of buttons
				amountArray = new Button[drugCount];
				priceArray = new Button[drugCount];
				// Iterate through all the drugs and add the needed visuals.
				//   [Quanity] [Drug Name] [Price]
				Iterator drugSettingIter = drugSettingList.iterator();
				while(drugSettingIter.hasNext()) {
					tempDrugSetting = (DrugSettingsD) drugSettingIter.next();

					// First Build the Sell Buttons
					tempButton = new Button("-1");
					amountArray[Kounter] = tempButton;
					tempButton.addActionListener(new sellListener(Kounter));
					add(tempButton);

					// Add the Label with the drug name
					add(new Label(tempDrugSetting.getName(), Label.CENTER));

					// Add the Buy Button
					tempButton = new Button("-1");
					priceArray[Kounter] = tempButton;
					tempButton.addActionListener(new buyListener(Kounter));
					add(tempButton);

					Kounter++;
				}

				// Add the Amount field
				add(new Label("Quantity to Sell/Buy:"));

				amountField = new TextField();
				add(amountField);

				// Add Some Static Data
				moneyLabel = new Label("Cash: NA");
				add(moneyLabel);
				pocketLabel = new Label("Pockets: 0/0");
				add(pocketLabel);
				loanLabel = new Label("Loan: NA");
				add(loanLabel);

				tempButton = new Button("Next Turn");
				tempButton.addActionListener(this);
				add(tempButton);
			}

			public void update(LocationD location) {
				DrugD tempDrug;

				currLocation = location;
				ArrayList drugList = location.getDrugList();
				int Kounter = 0;
				Iterator drugListIter = drugList.iterator();
				while(drugListIter.hasNext()) {
					tempDrug = (DrugD) drugListIter.next();
					priceArray[Kounter].setLabel("" + tempDrug.getCurrentPrice());
					amountArray[Kounter].setLabel("" + m_user.getDrugAmount(Kounter));
					Kounter++;
				}
				moneyLabel.setText("Cash: " + m_user.getMoney());
				pocketLabel.setText("Pockets: " + m_user.getFilledPockets() + "/"
														+ m_user.getPockets());
				loanLabel.setText("Loan: " + m_user.getLoanMoney());
			}

			public void actionPerformed(ActionEvent ae) {
				// Next Turn Button Clicked
				turnOver();
			}

			public class buyListener implements ActionListener {
				int m_drugID;

				public buyListener(int drugID) {
					m_drugID = drugID;
				}

				public void actionPerformed(ActionEvent ae) {
					ArrayList drugList = currLocation.getDrugList();
					String ac = ae.getActionCommand();
					// System.out.println("DrugID: " + m_drugID + ", amt: " + ac);
					try {
						m_user.buy(m_drugID, (DrugD) drugList.get(m_drugID), amountField.getText());
					} catch (NumberFormatException e) {
						// BUGBUG Non integer amount entered
					} catch (InsufficientFundsX e) {
						// BUGBUG Not enough money
					} catch (InsufficientSpaceX e) {
						// BUGBUG Not enough pockets
					} catch (DrugUnavailableX e) {
					} finally {
						update(currLocation);
					}
				}
			}

			public class sellListener implements ActionListener {
				int m_drugID;

				public sellListener(int drugID) {
					m_drugID = drugID;
				}

				public void actionPerformed(ActionEvent ae) {
					ArrayList drugList = currLocation.getDrugList();
					String ac = ae.getActionCommand();
					// System.out.println("DrugID: " + m_drugID + ", amt: " + ac);
					try {
						m_user.sell(m_drugID, (DrugD) drugList.get(m_drugID), amountField.getText());
					} catch (NumberFormatException e) {
						// BUGBUG Non integer amount entered
					} catch (InsufficientQuantityX e) {
						// BUGBUG Not enough dope
					} catch (DrugUnavailableX e) {
					} finally {
						update(currLocation);
					}
				}
			}

		}


		/**
		 * The panel that allows a user to find a location
		 */
		private class LocationListPanel extends Panel implements ActionListener {
			public String name = "LocationListPanel";
			public LocationListPanel() {
				// All right, we built a happy panel from the array list
				Button tempButton;
				ArrayList locationList = m_gameSettings.getLocationList();
				Iterator locationIter = locationList.iterator();
				while(locationIter.hasNext()) {
					String locationName = ((LocationD) locationIter.next()).getName();
					tempButton = new Button(locationName);
					tempButton.addActionListener(this);
					add(tempButton);
				}
			}
			public void actionPerformed(ActionEvent ae) {
				String ac = ae.getActionCommand();
				gotoLocation(ac);
			}
		}

    /**
     * The user settings panel is the first panel responsible for getting
     * information from the user.  It must determine connection information
     * as well as user settings.
		 * @author Jordan Reed
		 * @version 1.0
		 * @since Phase 2.0 (Meat)
     */
    private class UserSettingsPanel extends Panel implements ActionListener {
			private String CONNECTBUTTON = "Connect";
			TextField serverLocation;
			TextField userName;
			public String name = "UserSettingsPanel";

			public UserSettingsPanel() {
				GridBagLayout gbl = new GridBagLayout();
				GridBagConstraints gc = new GridBagConstraints();
				Button b;
				// We need to add information to get the server as well as the
				// User Settings


				gc.weightx = 1;
				gc.weighty = 0;
				gc.anchor = GridBagConstraints.NORTH;
				gc.insets = new Insets(2,2,2,2);

				setLayout(gbl);

				gc.gridx = 0;
				gc.gridy = 0;
				gc.gridwidth = 2;
				add(new Label("User Settings"));
				gc.gridwidth = 1; // Cleanup

				gc.gridx = 0;
				gc.gridy = 1;
				add(new Label("Server:"), gc);
				serverLocation = new TextField("localhost", 30);
				gc.gridx = 1;
				gc.gridy = 1;
				add(serverLocation, gc);

				gc.gridx = 0;
				gc.gridy = 2;
				add(new Label("User Name:"), gc);
				userName = new TextField(20);
				gc.gridx = 1;
				gc.gridy = 2;
				add(userName, gc);

				gc.gridx = 0;
				gc.gridy = 3;
				gc.weighty = 1;
				gc.gridwidth = 2;
				b = new Button(CONNECTBUTTON);
				add(b, gc);

				b.addActionListener(this);
			}

			public void actionPerformed(ActionEvent ae) {
				String ac = ae.getActionCommand();
				if (ac.equals(CONNECTBUTTON)) {
					System.out.println("Connect button pressed.");
					System.out.println("Server " + serverLocation.getText());
					System.out.println("Name " + userName.getText());
					//BUGBUG Check if Server/Name are empty.

					showWaitingOnServer("Making Initial Server Request");

					m_userSettings = new UserSettingsD(userName.getText());

					try {
						connectWithServer(serverLocation.getText());
					} catch (IOException e) {
						showComponent(m_userSettingsPanel.name);
					}
			}
		}
	} // End UserSettingsPanel

	private class ScorePanel extends Panel {
		public String name = "ScorePanel";

		public ScorePanel(ScoreListD scoreList) {
			int playerKount = scoreList.getPlayerCounter();

			setLayout(new GridLayout(playerKount + 1, 2));
			add(new Label("Player Name"));
			add(new Label("Score"));

			for(int Kounter = 0; Kounter < playerKount; Kounter++) {
				add(new Label(scoreList.getPlayerName(Kounter)));
				add(new Label("" + scoreList.getPlayerScore(Kounter)));
			}
		}
	}
}
