// Source file: c:/DopeWars/DopeWars/DopeServer.java

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

import DopeWars.DopeWars.*;
import DopeWars.DopeWars.DataHolders.*;
import DopeWars.Framework.*;


/**
 * This is the dope server.  It is responsible for coordinating a game of
 * DopeWars.  All clients will attach to this server.
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Phase 2.0 (Meat)
 */
public class DopeServer extends Frame implements IConnectorUser,
																						     IRequestTypes,
																								 WindowListener {
	// The three panels contained
	private ServerSettingsPanel m_serverSettingsPanel;
	private DebugPanel m_debugPanel;
	private StatusPanel m_statusPanel;

	// The Various Data Objects
	private CurrentStateD m_currentState;
	private GameSettingsD m_gameSettings;
	private ScoreListD m_scoreList;
	private MultiConnector m_multiConnector;

	/**
	 * Instantiates a new instance of the DopeServer class.  Then sets the
	 * appropriate size and location before displaying it on the screen.
	 *
	 * @roseuid 3924CA18001B
	*/
	public static void main(String args[]) {
		Frame f = new DopeServer();
		f.setSize(640,480);
		f.setLocation(0,0);
		f.setVisible(true);
	}

	/**
	 * The constructor method must create the three panels of the server
	 * and add them to the main frame.
	 *
	 * @roseuid 3924CCD401F8
	*/
	public DopeServer() {
		super("Dope Server");

		// Initialize all of the member variables
		m_currentState = new CurrentStateD();

		// Initialize All Of the Visuals
		setLayout(new GridLayout(3,1));

		m_serverSettingsPanel = new ServerSettingsPanel();
		add(m_serverSettingsPanel);

		m_statusPanel = new StatusPanel();
		add(m_statusPanel);

		m_debugPanel = new DebugPanel();
		add(m_debugPanel);

		addWindowListener(this);

		// Now that the graphical portion is done we should initiate the connector
		try {
			ServerSocket ss = new ServerSocket(4440);
			m_multiConnector = new MultiConnector(ss, this);
			m_debugPanel.addText("MultiConnector Created");
		} catch (IOException e) {
			m_debugPanel.addText("Error creating MultiConnector");
		}
	}

	/**
	 * @roseuid 3924CCDB0361
	*/
    private void debug(String s) {
    }

    /**
       @roseuid 3925F5AF0172
     */
    public void handleRequest(int requestType, Object requestObject, String requestor) {
			m_debugPanel.addText(requestor + ": Request Made.  Type: " + requestType);
			switch (requestType) {
				case HELLO:
					// A new client has attached and is issueing the greeting.
					addUser( (UserSettingsD) requestObject, requestor);
					break;
				case GET_LOCATION:
					// Give the client the requested location
					sendLocation( (String) requestObject, requestor);
					break;
				case TURN_DONE:
					// A client has finished his turn
					addTurnOver( (UserD) requestObject, requestor);
					break;
				case GAME_DONE:
				  // A client has finished the game
					addFinishedUser( (UserD) requestObject, requestor);
					break;
			}
    }

    public void closeConnection(String requestor) {
			m_debugPanel.addText(requestor + ": Connection Closed.");
			// Now we need to remove the user from the current state so they
			// don't get information anymore.
			m_currentState.removeUser(requestor);
			m_statusPanel.update();
		}

    /**
     * This method is called when a new user decides to join the game.
     * This only works when the game state is waiting for new people to join.
     * Otherwise the user is given the boot.
     *
     * @param user The information for the user to be added
     * @since Phase 2.0 (Meat)
     * @roseuid 39260A7201B0
     */
    private void addUser(UserSettingsD user, String requestor) {
			// This needs to add the user to the current state.
			m_currentState.addUser(user, requestor);
			m_statusPanel.update();
    }

    /**
     */
    private void startGame() {
			m_debugPanel.addText("Game being started");
			m_gameSettings = new GameSettingsD();
			// Set anything special from the settings panel
			m_multiConnector.send(GAME_START, m_gameSettings);
			m_currentState.newGame(m_gameSettings);
			nextTurn();
    }

    /**
       @roseuid 3926D17E0101
     */
    private void sendLocation(String locationName, String requestor) {
			LocationD tempLocation = m_currentState.getLocation(locationName);
			m_debugPanel.addText(requestor + ": Sending " + tempLocation.getName());
			m_multiConnector.send(GIVE_LOCATION, tempLocation, requestor);
    }

    /**
       @roseuid 3926DFBC01E0
     */
    private void nextTurn() {
			m_debugPanel.addText("Next turn issued");
			m_currentState.nextTurn();
			if(m_currentState.isGameOver()) {
				gameOver();
			} else {
				m_multiConnector.send(NEXT_TURN, "" + m_currentState.getTurn());
				m_statusPanel.update();
			}
    }

    /**
       @roseuid 3926DFFF03A9
     */
    private void updateState() {
    }

    /**
       @roseuid 392737ED0181
     */
    private void addTurnOver(UserD user, String requestor) {
			m_debugPanel.addText(requestor + ": Turn Over");
			m_currentState.turnDone(requestor);
			m_statusPanel.update();
			// Is everyone done?
			if(m_currentState.isTurnDone()) {
				m_debugPanel.addText("All users are done");
				nextTurn();
			}
    }

    private void addFinishedUser(UserD user, String requestor) {
			m_debugPanel.addText(requestor + ": Game Over");
			m_currentState.gameDone(requestor);
			m_scoreList.addUser(user);
			if(m_currentState.isGameDone()) {
				m_debugPanel.addText(requestor + ": All users are done with game");
				m_multiConnector.send(SCORE_LIST, m_scoreList);
			}
		}

    /**
       @roseuid 39273A3702F6
     */
    private void gameOver() {
			m_debugPanel.addText("Game Over");
			m_scoreList = new ScoreListD();
			m_multiConnector.send(GAME_OVER, "GAMEOVER");
    }

    public void windowClosing(WindowEvent we) {
			System.exit(0);
		}
    public void windowClosed(WindowEvent we) {}
    public void windowIconified(WindowEvent we) {}
    public void windowActivated(WindowEvent we) {}
    public void windowOpened(WindowEvent we) {}
    public void windowDeiconified(WindowEvent we) {}
    public void windowDeactivated(WindowEvent we) {}

	/**
	 * This panel contains settings for the game
	 *
	 * @author Jordan Reed
	 * @version 1.0
	 * @since Phase 2.0 (Meat)
	 */
	private class ServerSettingsPanel extends Panel implements ActionListener {
		public String s_startString = "Start Game";
		private Button m_startButton;
		public ServerSettingsPanel() {
			m_startButton = new Button(s_startString);
			add(m_startButton);
			m_startButton.addActionListener(this);
		}

		public void actionPerformed(ActionEvent ae) {
			String ac = ae.getActionCommand();
			if (ac.equals(s_startString)) {
				System.out.println("Game Start");
				m_startButton.setEnabled(false);
				startGame();
			}
		}
	}

	/**
	 * This panel contains status and debug information.
	 *
	 * @author Jordan Reed
	 * @version 1.0
	 * @since Phase 2.0 (Meat)
	 */
	private class DebugPanel extends Panel {
		/**
		 * This holds the text box where information will be added.
		 *
		 * @since Phase 2.0 (Meat)
		 */
		private TextArea m_debugText;

		/**
		 * This is the main constructor.  It must create the text area and add
		 * it to the panel.
		 *
		 * @author Jordan Reed
		 * @version 1.0
		 * @since Phase 2.0 (Meat)
		 */
		public DebugPanel() {
			m_debugText = new TextArea("", 8, 80); //, SCROLLBARS_BOTH);
			m_debugText.setEditable(false);
			m_debugText.setText("Debug Panel Loaded\n");
			add(m_debugText);
		}

		/**
		 * Adds text to be displayed on the screen.
		 *
		 * @author Jordan Reed
		 * @version 1.0
		 * @since Phase 2.0 (Meat)
		 */
		public void addText(String newText) {
			m_debugText.setText(newText + "\n" + m_debugText.getText());
		}
	}

	/**
	 * This panel contains information about the game status.  Such as
	 * what turn it is, and who is playing.
	 *
	 * @author Jordan Reed
	 * @version 1.0
	 * @since Phase 2.0 (Meat)
	 */
	private class StatusPanel extends Panel {
		/**
		 * This is a graphical list that will show all of the people currently
		 * connected to the server
		 *
		 * @since Phase 2.0 (Meat)
		 */
		java.awt.List m_userList;
		Label turnLabel;


		/**
		 * The public constructor.  This will add all of the components to the
		 * panel.
		 *
		 * @since Phase 2.0 (Meat)
		 */
		public StatusPanel() {
			m_userList = new java.awt.List(5, false);
			add(m_userList);
			turnLabel = new Label("Turn: 0");
			add(turnLabel);
		}

		/**
		 * The update method will update the status window to reflect the current
		 * state of the game.
		 */
		public void update() {
			// First this will will empty the user list and rebuild it.
			int Kounter = 0;
			m_userList.removeAll();
			ArrayList tempPlayerList = m_currentState.getPlayerList();
			Iterator playerIter = tempPlayerList.iterator();
			while(playerIter.hasNext()) {
				UserD user = (UserD) playerIter.next();
				m_userList.add(user.getName() + " " + m_currentState.getPlayersStatus(Kounter));
				Kounter++;
			}
			turnLabel.setText("Turn: " + m_currentState.getTurn());
		}
	}
}

