// Source file: c:/DopeWars/DopeWars/DataHolders/CurrentStateD.java

package DopeWars.DopeWars.DataHolders;

import DopeWars.DopeWars.*;
import java.util.*;

/**
 * This class contains all of the information about the current state of
 * the game that is occuring.
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Phase 2.0 (Meat)
 */

public class CurrentStateD implements IPlayerStatus{
	private ArrayList m_players;
	private ArrayList m_playersStatus;
	private int m_turnNumber = 0;
	private GameSettingsD m_gameSettings;
	private HashMap m_locationMap;

	public CurrentStateD() {
		m_players = new ArrayList();
		m_playersStatus = new ArrayList();
	}

	/**
	 * Based on a UserSettingsD object, this will create a new user object
	 * which will then be added to the current state of the game.
	 *
	 * @param currUser The user to be added to the state
	 * @since Phase 2.0 (Meat)
 	 */
	public void addUser(UserSettingsD currUser, String requestor) {
		String tempName;
		UserD tempUser;

		tempName = currUser.getName();
		tempUser = new UserD(tempName, requestor);
		m_players.add(tempUser);
		m_playersStatus.add(new Integer(READY_TO_START));
	}

	/**
	 * This remove a user from the current state based on their requestor
	 *
	 * @since Phase 2.0 (Meat)
 	 */
	public void removeUser(String requestor) {
		System.out.println("removing user");
		System.out.println("making iterator");
		int Kounter = 0;
		Iterator playerIter = m_players.iterator();
		while(playerIter.hasNext()) {
			UserD user = (UserD) playerIter.next();
			System.out.println("Checking equality of " + user.getID() + " and " + requestor);
			if(user.getID().equals(requestor)) {
				playerIter.remove();
				m_playersStatus.remove(Kounter);
			}
			Kounter++;
		}
	}

	/**
	 * This returns the ArrayList of the current players
	 *
	 * @return The list of current players
	 * @since Phase 2.0 (Meat)
	 */
	public ArrayList getPlayerList() {
		return m_players;
	}

	public int getPlayersStatus(int playerID) {
		return ((Integer) m_playersStatus.get(playerID)).intValue();
	}

	public void newGame(GameSettingsD currentSettings) {
		m_locationMap = new HashMap();
		m_gameSettings = currentSettings;
		// Flesh out the current State
		ArrayList locationList = m_gameSettings.getLocationList();
		ArrayList drugSettingList = m_gameSettings.getDrugSettingList();
		LocationD tempLocation;

		Iterator locationIter = locationList.iterator();
		while(locationIter.hasNext()) {
			tempLocation = (LocationD) locationIter.next();
			tempLocation.setDrugList(makeDrugList(drugSettingList));
			// Okay, now added it to the map
			m_locationMap.put(tempLocation.getName(), tempLocation);
		}
	}

	private ArrayList makeDrugList(ArrayList drugSettingList) {
		ArrayList drugList = new ArrayList();
		DrugSettingsD tempDrugSettings;

		Iterator drugSettingIter = drugSettingList.iterator();
		while(drugSettingIter.hasNext()) {
			tempDrugSettings = (DrugSettingsD) drugSettingIter.next();
			drugList.add(new DrugD(tempDrugSettings));
		}
		return drugList;
	}

	public void nextTurn() {
		// This needs to go through and update ALL the drug prices!
		LocationD tempLocation;
		DrugD tempDrug;
		ArrayList drugList;

		m_turnNumber ++;
		Set locationSet = m_locationMap.entrySet();
		Iterator locationIter = locationSet.iterator();
		while(locationIter.hasNext()) {
			Map.Entry me = (Map.Entry) locationIter.next();
			tempLocation = (LocationD) me.getValue();
			drugList = tempLocation.getDrugList();
			Iterator drugIter = drugList.iterator();
			while(drugIter.hasNext()) {
				tempDrug = (DrugD) drugIter.next();
				tempDrug.update();
			}
		}

		// We need to update all the users to taking a turn
		m_playersStatus.clear();
		for(int Kounter = 0; Kounter < m_players.size(); Kounter++) {
			m_playersStatus.add(new Integer(TAKING_TURN));
		}
	}

	public void turnDone(String requestor) {
		updateStatus(requestor, TURN_OVER);
	}

	public void gameDone(String requestor) {
		updateStatus(requestor, GAME_OVER);
	}

	private void updateStatus(String requestor, int status) {
		int Kounter = 0;
		UserD tempUser;
		Iterator playerIter = m_players.iterator();
		while(playerIter.hasNext()) {
			tempUser = (UserD) playerIter.next();
			if( (tempUser.getID()).equals(requestor) ) {
				m_playersStatus.remove(Kounter);
				m_playersStatus.add(Kounter, new Integer(status));
			}
			Kounter++;
		}
	}


	public boolean isTurnDone() {
		// Iterate through all the users to see if they are done
		int status;
		Iterator playerStatusIter = m_playersStatus.iterator();
		while(playerStatusIter.hasNext()) {
			status = ((Integer) playerStatusIter.next()).intValue();
			if (status != TURN_OVER) {
				return false;
			}
		}
		return true;
	}

	public boolean isGameDone() {
		// Iterate through all the users to see if they are done
		int status;
		Iterator playerStatusIter = m_playersStatus.iterator();
		while(playerStatusIter.hasNext()) {
			status = ((Integer) playerStatusIter.next()).intValue();
			if (status != GAME_OVER) {
				return false;
			}
		}
		return true;
	}

	public boolean isGameOver() {
		if(m_turnNumber > m_gameSettings.getTurns()) {
			return true;
		}
		return false;
	}

	public LocationD getLocation(String locationName) {
		return (LocationD) m_locationMap.get(locationName);
	}

	public int getTurn() {
		return m_turnNumber;
	}
}
