// Source file: c:/DopeWars/DopeWars/DataHolders/UserD.java

package DopeWars.DopeWars.DataHolders;

import DopeWars.DopeWars.Exceptions.*;
import java.util.*;
import java.io.*;

public class UserD implements Serializable {
    private int[] m_drugAmountList;
    private String m_name;
    private String m_ID;
    private int m_pockets = 0;
    private int m_money = 0;
    private int m_bankMoney = 0;
    private int m_loanMoney = 0;

    public UserD (String name, String requestor) {
			m_name = name;
			m_ID = requestor;
		}

    public UserD (UserSettingsD userSettings, GameSettingsD gameSettings) {
			m_name = userSettings.getName();
			m_pockets = gameSettings.getStartPockets();
			m_money = gameSettings.getStartMoney();
			m_bankMoney = gameSettings.getStartBankMoney();
			m_loanMoney = gameSettings.getStartLoanMoney();
			m_drugAmountList = new int[gameSettings.getNumberOfDrugs()];
		}

		public String getName() {
			return m_name;
		}

		public String getID() {
			return m_ID;
		}

		public int getDrugAmount(int drugNumber) {
			return m_drugAmountList[drugNumber];
		}

		public int getMoney() {
			return m_money;
		}

		public int getPockets() {
			return m_pockets;
		}

		public int getLoanMoney() {
			return m_loanMoney;
		}

		public int getFilledPockets() {
			int total = 0;
			for(int i = 0; i < m_drugAmountList.length; i++) {
				total += m_drugAmountList[i];
			}

			return total;
		}

		public int getEmptyPockets() {
			return (m_pockets - getFilledPockets());
		}

		public void buy(int drugID, DrugD drug, String amount) throws NumberFormatException,
																											InsufficientFundsX,
																											InsufficientSpaceX,
																											DrugUnavailableX {
			int intAmount;
			intAmount = Integer.parseInt(amount);
			buy(drugID, drug, intAmount);
		}

		public void buy(int drugID, DrugD drug, int amount) throws InsufficientFundsX,
																									 InsufficientSpaceX,
																									 DrugUnavailableX {
			int cost = amount * drug.getCurrentPrice();
			if (drug.getCurrentPrice() < 0) {
				System.out.println("Not available");
				throw new DrugUnavailableX();
			} else if(cost > m_money) {
				// Do we have enough money
				System.out.println("Not Enough Money!");
			  throw new InsufficientFundsX();
			} else if (amount > getEmptyPockets()) {
				System.out.println("Not Enough Space!");
				throw new InsufficientSpaceX();
			}
			System.out.println(m_name + " buying " + amount + " of " + drug.getName() + " for " + cost);

			m_money -= cost; // Take Away Money
			m_drugAmountList[drugID] += amount; // Add Drug
		}

		public void sell(int drugID, DrugD drug, String amount) throws NumberFormatException,
																											DrugUnavailableX,
																											InsufficientQuantityX {
			int intAmount;
			intAmount = Integer.parseInt(amount);
			sell(drugID, drug, intAmount);
		}

		public void sell(int drugID, DrugD drug, int amount) throws DrugUnavailableX,
																																InsufficientQuantityX {
			int cost = amount * drug.getCurrentPrice();
			if (drug.getCurrentPrice() < 0) {
				System.out.println("Not available");
				throw new DrugUnavailableX();
			} else if (amount > m_drugAmountList[drugID]) {
				System.out.println("Not enough");
				throw new InsufficientQuantityX();
			}
			System.out.println(m_name + " selling " + amount + " of " + drug.getName() + " for " + cost);

			m_money += cost; // Add Money
			m_drugAmountList[drugID] -= amount; // Remove Drug
		}

		public void nextTurn(GameSettingsD gameSettings) {
			// Next Turn.  Let's add up fines
			m_loanMoney *= gameSettings.getLoanInterest();
		}
}
