// Source file: c:/DopeWars/Framework/IConnectorUser.java

package DopeWars.Framework;

/**
 * This should be implemented by any class which wishes to have a connector
 * and be able to receive information from it.
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Phase 1.0 (Framework)
*/
public interface IConnectorUser {

    /**
     *  When a connector receives a new packet it will call this method on
     *  it's user to allow it to handle the response.
     *
     *  @param requestType The type of packet being send
     *  @param requestObject An object to be sent with the packet
     *  @param requestor The name of the requestor on the other side.  This is used for a server to know which client is requesting service.
     *  @since Phase 1.0 (Framework)
     *  @roseuid 3925FA0C0150
     */
    public void handleRequest(int requestType, Object requestObject, String requestor);

    /**
     *  When a connector looses it connection to the other side it will call
     *  this method.
     *
     *  @param requestor The name of the requestor on the other side.  This is used for a server to know which client is requesting service.
     *  @since Phase 1.0 (Framework)
     *  @roseuid 3925FA0C0150
     */
    public void closeConnection(String requestor);
}
