// Source file: c:/DopeWars/Framework/Packet.java

package DopeWars.Framework;

import java.io.*;

/**
 * This is the basic unit of information transmitted between connectors.
 * A packet, based on an inputstream or output stream is intelligent
 * enough to send/receive it's information for you.
 *
 * @author Jordan Reed
 * @see Connector
 * @version 1.0
 * @since Phase 1.0 (Framework)
*/

public class Packet {
	public static final String START = "Start";
	public static final String STOP = "Stop";

	/**
   * Holds the value of the packet type.  This is held in the object
	 * wrapper for Integer.
	 *
	 * @since Phase 1.0 (Framework)
	 */
	private Integer m_packetType;

	/**
	 * Holds the value of the object received by the packet.
	 *
	 * @since Phase 1.0 (Framework)
	 */
	private Object m_thing;

	/**
	 * Receiving constructor.  This will read the next packet of information
	 * out of the input stream and store it in the member variables.
	 *
	 * @param in The InputStream to read the packet from
	 * @throws SocketException
	 * @since Phase 1.0 (Framework)
	 * @roseuid 3925FDD801E4
	 */
	public Packet(InputStream in) throws IOException {
		System.out.println("Creating Input Packet");
		String tempString;

    try {
			int c;
			ObjectInputStream ois = new ObjectInputStream(in);

			// Read in the Header
			tempString = (String) ois.readObject();
			if( ! tempString.equals(START) ) {
				throw new IllegalArgumentException();
			}

			// Read in the Type
			m_packetType = (Integer) ois.readObject();

			// Read in the Thing
			m_thing = (Object) ois.readObject();

			// Read in the Footer
			tempString = (String) ois.readObject();
			if( ! tempString.equals(STOP) ) {
				throw new IllegalArgumentException();
			}

			// System.out.println(m_thing.toString());
		} catch (IOException e) {
			System.out.println("e:" + e);
			throw e;
		} catch (ClassNotFoundException e) {
			System.out.println("f:" + e);
		} finally {
			System.out.println("Input Packet Done");
		}
	}

	/**
	 * Sending construct.  This will encapsulate the type and object and
	 * send it over the output stream.
	 *
	 * @param out The output stream to send the packet over
	 * @param packetType The type of packet being sent
	 * @param thing The object being sent.
	 * @roseuid 3925FEB8036C
	 */
	public Packet(OutputStream out, int packetType, Object thing) {
		m_packetType = new Integer(packetType);
		m_thing = thing;

		// Open Connection with Server
		System.out.println("Creating Streams");
		try {
			ObjectOutputStream oos = new ObjectOutputStream(out);

			// Send the Start Packet
			oos.writeObject(START);

			// Send the Packet Type
			oos.writeObject(m_packetType);

			// Send the Object
			oos.writeObject(m_thing);

			// End it
			oos.writeObject(STOP);

			System.out.println("Sent it all...");
		} catch (IOException e) {
			System.out.println("IOException:" + e);
		}
	}

    /**
     * Gets the thing object from a recieved packet.
     *
     * @return The object associated with packet
     * @roseuid 3925FEFC034C
     */
    public Object getThing() {
			return m_thing;
    }

    /**
     * Gets the type from a recieved packet
     *
     * @return The type of packet received
     * @roseuid 3925FF0E00B3
     */
    public int getType() {
			return m_packetType.intValue();
    }
}
