// Source file: c:/DopeWars/Framework/TurnThread.java

package DopeWars.Framework;

/**
 * This object is responsible for tracking a turn an notifiying the
 * TurnUser object when a turn has ended.
 *
 * @author Jordan Reed
 * @see ITurnUser
 * @version 1.0
 * @since Phase 1.0 (Framework)
 */

public class TurnThread implements Runnable {
	  /**
	   * Holds a reference to the object that should be notified when the
	   * turn is over.
	   *
	   * @since Phase 1.0 (Framework)
	   */
	  ITurnUser m_turnUser;

	  /**
	   * Holds the length of time before the turn has ended.
	   *
	   * @since Phase 1.0 (Framework)
	   */
	  int m_turnLength;

    /**
     * The constructor stores the member objects and executes the thread.
     *
     * @since Phase 1.0 (Framework)
     * @param turnUser The object that should be notified when the turn is ub
     * @param turnLength The length of the turn
     * @roseuid 392735DA01D7
     */
    public TurnThread(ITurnUser turnUser, int turnLength) {
			System.out.println("TurnThread()");
			m_turnUser = turnUser;
			m_turnLength = turnLength;
			(new Thread(this, "Turn Thread")).start();
    }

		/**
		 * This method will cause the object to sleep for the length of the turn
		 * and then notify the turn using object.
		 */
    public void run() {
			System.out.println("run()");
			try {
				Thread.sleep(m_turnLength * 1000);
			} catch(InterruptedException e) {
				// Do nothing.  It's all done in the finally section.
			} finally {
				m_turnUser.turnOver();
			}
		}
}
